/*
 * Copyright (c) 2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/** @file */

#ifndef _IVMOB_H
#define _IVMOB_H

#include <stdbool.h>
#include <stdint.h>
/** \defgroup macros Macro definitions */
/**@{*/

/** @name Response bitfield decoding */
#define IVMOB_ACTIVE_MODE_BIT		0x02 /**< 0 -> active */
#define IVMOB_COMMAND_SELECTION_BIT	0x01 /**< 0 -> local 1 -> distant */

#define IVMOB_LINEAR_SPEED_OK_BIT	0x02 /**< 1 -> speed too high */
#define IVMOB_ANGULAR_SPEED_OK_BIT	0x01 /**< 1 -> speed too high */

#define IVMOB_MMI_MOTION_ENABLE_BIT	0x04 /**< 0 -> enabled 1 disabled */
#define IVMOB_MMI_CHARGE_CONNECTOR_BIT	0x02 /**< 1 -> plugged */
#define IVMOB_MMI_COMMAND_SELECTION_BIT	0x01 /**< 0 -> local 1 -> distant */

/** @name Command origin */
#define IVMOB_CMD_ORIGIN_LOCAL		0x00 /**< 0 -> load */
#define IVMOB_CMD_ORIGIN_REMOTE		0x01 /**< 1 -> remote control */

/** @name Operating modes */
#define IVMOB_MODE_INACTIVE		0x01 /**< inactive motion impossible */
#define IVMOB_MODE_ACTIVE		0x02 /**< active motion possible */
#define IVMOB_MODE_SAFETY		0x03 /**< safety emergency stop */
#define IVMOB_MODE_WAIT_RESET		0x04 /**< waiting for reset button */

/** @name Radio buttons */
#define IVMOB_RADIO_BUTTON_LEFT		0x80
#define IVMOB_RADIO_BUTTON_ON		0x40
#define IVMOB_RADIO_BUTTON_NEXT		0x20
#define IVMOB_RADIO_BUTTON_PREV		0x10
#define IVMOB_RADIO_BUTTON_4		0x08
#define IVMOB_RADIO_BUTTON_3		0x04
#define IVMOB_RADIO_BUTTON_2		0x02
#define IVMOB_RADIO_BUTTON_1		0x01

/** @name Safety error codes */
#define IVMOB_ERROR_FRONT_ESTOP		0x01
#define IVMOB_ERROR_REAR_ESTOP		0x02
#define	IVMOB_ERROR_RADIO_ESTOP		0x03
#define IVMOB_ERROR_PAYLOAD_ESTOP	0x04
#define IVMOB_ERROR_FRONT_BUMPER	0x05
#define IVMOB_ERROR_REAR_BUMPER		0x06
#define IVMOB_ERROR_FRONT_LASER_SCANNER	0x07
#define IVMOB_ERROR_REAR_LASER_SCANNER	0x08
#define IVMOB_ERROR_MOTOR_PROBLEM	0x40
#define IVMOB_ERROR_INTERNAL_COMM	0x41
#define IVMOB_ERROR_PAYLOAD_COMM	0x42
#define IVMOB_ERROR_INTERNAL_ERROR	0x43
#define IVMOB_ERROR_BATTERY_SAFETY	0x80
#define IVMOB_ERROR_CHARGER_CONNECTED	0x81
#define IVMOB_ERROR_MOTION_DISABLED	0x82
#define IVMOB_ERROR_BREAKS_DISABLED	0x83
#define IVMOB_ERROR_MODE_CHANGED	0x84
#define IVMOB_ERROR_INITIALIZING	0xf0
#define IVMOB_ERROR_OK			0xff
/**@}*/
/**@}*/

/**
 * @brief Internal state of the robot
 */
struct ivmob {
	struct ivmob_private *priv;
	int16_t v;			/**< 10e-3 m/s */
	int16_t w;			/**< 10e-3 rad/s */
	int8_t cmd_origin;		/**< 0 local - 1 remote */
	int32_t front_left_wheel_speed; /**< 10e-4 rad/s */
	int32_t front_right_wheel_speed; /**< 10e-4 rad/s */
	int32_t rear_left_wheel_speed;	 /**< 10e-4 rad/s */
	int32_t rear_right_wheel_speed;  /**< 10e-4 rad/s */
	int32_t front_left_current; 	 /**< 10e-3 A */
	int32_t front_right_current;	 /**< 10e-3 A */
	int32_t rear_left_current;	 /**< 10e-3 A */
	int32_t rear_right_current;	 /**< 10e-3 A */
	uint8_t temperature;		 /**< deg Celsius */
	uint8_t battery_level;		 /**< 0: empty .. 8: full */
	uint8_t mmi;		   /**< cf IVMOB_MMI_ bits above */
	uint8_t safety_error_code; /**< cf IVMOB_ERROR_ codes above */
	uint8_t operational_mode;  /**< cf IVMOB_MODE_ codes above */
	uint8_t radio_buttons;	   /**< cf IVMOB_RADIO_ bits above */
	uint8_t radio_page;
	uint8_t version_major;
	uint8_t version_minor;
	uint8_t version_tiny;
	uint8_t version_build;
	uint8_t commissioning_year; 	/**< last 2 digits */
	uint8_t commissioning_month; 	/**< 0x01 -> january */
	uint8_t commissioning_day;   	/**< 0x01 -> 1st day of month */
	uint32_t total_distance;     	/**< m */
	uint32_t total_on_time;	     	/**< min */
	uint16_t wheel_diameter;     	/**< mm */
	uint16_t torque_constant;    	/**< 10e-3 Nm/A rms */
	uint8_t gear_ratio;		/**< gear ratio */
	uint16_t wheel_track_width; 	/**< mm */
};

/** \defgroup functions Public functions */
/**@{*/
extern struct ivmob *ivmobInit(const char *, bool);
extern void ivmobEnd(struct ivmob *);
extern int ivmob_lock(struct ivmob *);
extern int ivmob_unlock(struct ivmob *);

extern int ivmobWait(struct ivmob *, int16_t, int);

extern int ivmobCmdSpeed(struct ivmob *, int16_t, int16_t, int);
extern int ivmobCmdActive(struct ivmob *, int);
extern int ivmobCmdInactive(struct ivmob *, int);
extern int ivmobRqstProcessedSpeed(struct ivmob *, int);
extern int ivmobRqstFrontWheelSpeed(struct ivmob *, int);
extern int ivmobRqstRearWheelSpeed(struct ivmob *, int);
extern int ivmobRqstFrontCurrent(struct ivmob *, int);
extern int ivmobRqstRearCurrent(struct ivmob *, int);
extern int ivmobRqstStatus(struct ivmob *, int);
extern int ivmobRqstSoftwareVersion(struct ivmob *, int);
extern int ivmobRqstCommissioningDate(struct ivmob *, int);
extern int ivmobRqstTotalDistance(struct ivmob *, int);
extern int ivmobRqstTotalOnTime(struct ivmob *, int);
extern int ivmobRqstWheelSize(struct ivmob *, int);
extern int ivmobRqstTorqueConstant(struct ivmob *, int);
extern int ivmobRqstGearRatio(struct ivmob *, int);
extern int ivmobRqstWheelTrackWidth(struct ivmob *, int);
extern int ivmobRqstPeriodicMessages(struct ivmob *,
	uint8_t, uint8_t, uint8_t, uint8_t, uint8_t, uint8_t, int);
extern int ivmobPayloadOffAck(struct ivmob *);

extern int ivmobReceiveAndDecode(struct ivmob *ivmob);

extern const char *ivmob_operating_mode(int);
extern const char *ivmob_safety_code(int);
/**@}*/
#endif
